package com.m2pool.manage.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.m2pool.common.core.Result.R;
import com.m2pool.common.core.constant.HttpStatus;
import com.m2pool.common.core.web.page.TableDataInfo;
import com.m2pool.manage.dto.ManageBroadcastDto;
import com.m2pool.manage.dto.ManageUserDto;
import com.m2pool.manage.mapper.ManageBroadcastMapper;
import com.m2pool.manage.service.ManageUserService;
import com.m2pool.manage.vo.ManageBaseVo;
import com.m2pool.manage.vo.ManageUserVo;
import com.m2pool.manage.vo.PageVo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;


/**
 * @Description 广播信息 服务实现类
 */
@Service
public class ManageUserServiceImpl implements ManageUserService {

    @Resource
    private ManageBroadcastMapper manageBroadcastMapper;

    @Resource
    private JavaMailSenderImpl javaMailSender;

    @Value("${spring.mail.username}")
    private String sendMailer;
    @Override
    public TableDataInfo<ManageUserDto> listInfo(ManageUserVo manageUserVo) {
        PageHelper.startPage(manageUserVo.getPageNum(), manageUserVo.getPageSize());
        List<ManageUserDto> manageUserDtos = manageBroadcastMapper.selectUserListByPage(manageUserVo.getCoin(), manageUserVo.getUser(), manageUserVo.getMinerUser());
        PageHelper.clearPage();
        return getDataTable(manageUserDtos);
    }

    private TableDataInfo<ManageUserDto> getDataTable(List<ManageUserDto> list)
    {
        TableDataInfo<ManageUserDto> rspData = new TableDataInfo<>();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setRows(list);
        rspData.setMsg("查询成功");
        PageInfo<ManageUserDto> pageInfo = new PageInfo<>(list);
        rspData.setTotal(pageInfo.getTotal());
        rspData.setTotalPage(pageInfo.getPages());
        return rspData;
    }


    @Override
    public void sendTextMailMessage(String to,String subject,String text){

        try {
            //true 代表支持复杂的类型
            MimeMessageHelper mimeMessageHelper = new MimeMessageHelper(javaMailSender.createMimeMessage(),true);
            //邮件发信人
            mimeMessageHelper.setFrom(sendMailer);
            //邮件收信人  1或多个
            mimeMessageHelper.setTo(to.split(","));
            //邮件主题
            mimeMessageHelper.setSubject(subject);
            //邮件内容
            mimeMessageHelper.setText(text);
            //邮件发送时间
            mimeMessageHelper.setSentDate(new Date());

            //发送邮件
            javaMailSender.send(mimeMessageHelper.getMimeMessage());
            //System.out.println("发送邮件成功："+sendMailer+"->"+to);

        } catch (Exception e) {
            e.printStackTrace();
            //System.out.println("发送邮件失败："+e.getMessage());
        }
    }
}
